@@ -3,9 +3,9 @@
  *
  * Implementation of the Tamper-Proof Device Filesystem.
  *
- * Copyright (C) 2005-2007  NTT DATA CORPORATION
+ * Copyright (C) 2005-2008  NTT DATA CORPORATION
  *
- * Version: 1.5.1   2007/10/19
+ * Version: 1.5.5   2008/09/03
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -36,8 +36,20 @@
 
 #include <linux/version.h>
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
 #define s_fs_info u.generic_sbp
+#else
+#include <linux/audit.h>
+#ifdef AUDIT_APPARMOR_AUDIT
+/* AppArmor patch adds "struct vfsmount" to VFS helper functions. */
+#define HAVE_VFSMOUNT_IN_VFS_HELPER
+#endif
+#endif
+
+#if defined(RHEL_MAJOR) && RHEL_MAJOR == 5
+#define HAVE_NO_I_BLKSIZE_IN_INODE
+#elif defined(AX_MAJOR) && AX_MAJOR == 3
+#define HAVE_NO_I_BLKSIZE_IN_INODE
 #endif
 
 /* The following constants are used to restrict operations.*/
@@ -86,7 +98,7 @@ static struct dentry *lookup_create2(const char *name, struct dentry *base, int
 {
 	struct dentry *dentry;
 	const int len = name ? strlen(name) : 0;
-#if LINUX_VERSION_CODE  >= KERNEL_VERSION(2,6,16)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 16)
 	mutex_lock(&base->d_inode->i_mutex);
 #else
 	down(&base->d_inode->i_sem);
@@ -108,7 +120,11 @@ static int fs_mkdir(const char *pathname, struct dentry *base, int mode, uid_t u
 	struct dentry *dentry = lookup_create2(pathname, base, 1);
 	int error = PTR_ERR(dentry);
 	if (!IS_ERR(dentry)) {
+#ifdef HAVE_VFSMOUNT_IN_VFS_HELPER
+		error = vfs_mkdir(base->d_inode, dentry, NULL, mode);
+#else
 		error = vfs_mkdir(base->d_inode, dentry, mode);
+#endif
 		if (!error) {
 			lock_kernel();
 			dentry->d_inode->i_uid = user;
@@ -117,7 +133,7 @@ static int fs_mkdir(const char *pathname, struct dentry *base, int mode, uid_t u
 		}
 		dput(dentry);
 	}
-#if LINUX_VERSION_CODE  >= KERNEL_VERSION(2,6,16)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 16)
 	mutex_unlock(&base->d_inode->i_mutex);
 #else
 	up(&base->d_inode->i_sem);
@@ -139,7 +155,11 @@ static int fs_mknod(const char *filename, struct dentry *base, int mode, dev_t d
 	dentry = lookup_create2(filename, base, 0);
 	error = PTR_ERR(dentry);
 	if (!IS_ERR(dentry)) {
+#ifdef HAVE_VFSMOUNT_IN_VFS_HELPER
+		error = vfs_mknod(base->d_inode, dentry, NULL, mode, dev);
+#else
 		error = vfs_mknod(base->d_inode, dentry, mode, dev);
+#endif
 		if (!error) {
 			lock_kernel();
 			dentry->d_inode->i_uid = user;
@@ -148,7 +168,7 @@ static int fs_mknod(const char *filename, struct dentry *base, int mode, dev_t d
 		}
 		dput(dentry);
 	}
-#if LINUX_VERSION_CODE  >= KERNEL_VERSION(2,6,16)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 16)
 	mutex_unlock(&base->d_inode->i_mutex);
 #else
 	up(&base->d_inode->i_sem);
@@ -162,10 +182,18 @@ static int fs_symlink(const char *pathname, struct dentry *base, char *oldname,
 	struct dentry *dentry = lookup_create2(pathname, base, 0);
 	int error = PTR_ERR(dentry);
 	if (!IS_ERR(dentry)) {
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0) && LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 26)
+#ifdef HAVE_VFSMOUNT_IN_VFS_HELPER
+		error = vfs_symlink(base->d_inode, dentry, NULL, oldname, S_IALLUGO);
+#else
 		error = vfs_symlink(base->d_inode, dentry, oldname, S_IALLUGO);
+#endif
+#else
+#ifdef HAVE_VFSMOUNT_IN_VFS_HELPER
+		error = vfs_symlink(base->d_inode, dentry, NULL, oldname);
 #else
 		error = vfs_symlink(base->d_inode, dentry, oldname);
+#endif
 #endif
 		if (!error) {
 			lock_kernel();
@@ -176,7 +204,7 @@ static int fs_symlink(const char *pathname, struct dentry *base, char *oldname,
 		}
 		dput(dentry);
 	}
-#if LINUX_VERSION_CODE  >= KERNEL_VERSION(2,6,16)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 16)
 	mutex_unlock(&base->d_inode->i_mutex);
 #else
 	up(&base->d_inode->i_sem);
@@ -429,7 +457,7 @@ static void MakeNode(struct dev_entry *entry, struct dentry *root)
 			struct dentry *new_base;
 			const int len = filename - name;
 			*filename = '\0';
-#if LINUX_VERSION_CODE  >= KERNEL_VERSION(2,6,16)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 16)
 			mutex_lock(&base->d_inode->i_mutex);
 			new_base = lookup_one_len(name, base, len);
 			mutex_unlock(&base->d_inode->i_mutex);
@@ -501,7 +529,7 @@ static int Syaoran_Initialize(struct super_block *sb, void *data)
 	static int first = 1;
 	if (first) {
 		first = 0;
-		printk("SYAORAN: 1.5.1   2007/10/19\n");
+		printk("SYAORAN: 1.5.5   2008/09/03\n");
 	}
 	{
 		struct inode *inode = new_inode(sb);
@@ -762,7 +790,7 @@ static void ReadTable(struct syaoran_read_struct *head, char *buf, int count)
 			len = snprintf(buf, count, "%-20s %3o %3u %3u %2u %c\n", name, perm, uid, gid, flags, 'f');
 			break;
 		}
-		if (len < 0 || count < len) break;
+		if (len < 0 || count <= len) break;
 		count -= len;
 		buf += len;
 		head->avail += len;
@@ -831,12 +859,14 @@ static int syaoran_create_tracelog(struct super_block *sb, const char *filename)
 			inode->i_mode = S_IFREG | 0400;
 			inode->i_uid = 0;
 			inode->i_gid = 0;
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 19)
+#ifndef HAVE_NO_I_BLKSIZE_IN_INODE
 			inode->i_blksize = PAGE_CACHE_SIZE;
+#endif
 #endif
 			inode->i_blocks = 0;
 			inode->i_mapping->a_ops = &syaoran_aops;
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0)
 			inode->i_mapping->backing_dev_info = &syaoran_backing_dev_info;
 			inode->i_op = &syaoran_file_inode_operations;
 #else
@@ -850,7 +880,7 @@ static int syaoran_create_tracelog(struct super_block *sb, const char *filename)
 		}
 		dput(dentry);
 	}
-#if LINUX_VERSION_CODE  >= KERNEL_VERSION(2,6,16)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 16)
 	mutex_unlock(&base->d_inode->i_mutex);
 #else
 	up(&base->d_inode->i_sem);
